@@ -11,7 +11,7 @@ mod ui;
 mod wasm_vm;
 
 use log::info;
-use std::collections::HashMap;
+use std::collections::{HashMap, HashSet};
 use std::{
     path::PathBuf,
     sync::{Arc, Mutex, RwLock},
@@ -49,11 +49,11 @@ use zellij_utils::{
     setup::get_default_data_dir,
 };
 
-pub(crate) type ClientId = u16;
+pub type ClientId = u16;
 
 /// Instructions related to server-side application
 #[derive(Debug, Clone)]
-pub(crate) enum ServerInstruction {
+pub enum ServerInstruction {
     NewClient(
         ClientAttributes,
         Box<CliArgs>,
@@ -134,9 +134,15 @@ impl SessionState {
         }
     }
     pub fn new_client(&mut self) -> ClientId {
-        let mut clients: Vec<ClientId> = self.clients.keys().copied().collect();
-        clients.sort_unstable();
-        let next_client_id = clients.last().unwrap_or(&0) + 1;
+        let clients: HashSet<ClientId> = self.clients.keys().copied().collect();
+        let mut next_client_id = 1;
+        loop {
+            if clients.contains(&next_client_id) {
+                next_client_id += 1;
+            } else {
+                break;
+            }
+        }
         self.clients.insert(next_client_id, None);
         next_client_id
     }
