@@ -12,6 +12,7 @@ use crate::{
     pty::{ClientOrTabIndex, PtyInstruction, VteBytes},
     tab::{Output, Tab},
     thread_bus::Bus,
+    ui::overlay::{Overlay, OverlayWindow, Overlayable},
     wasm_vm::PluginInstruction,
     ClientId, ServerInstruction,
 };
@@ -24,7 +25,7 @@ use zellij_utils::{
 
 /// Instructions that can be sent to the [`Screen`].
 #[derive(Debug, Clone)]
-pub(crate) enum ScreenInstruction {
+pub enum ScreenInstruction {
     PtyBytes(RawFd, VteBytes),
     Render,
     NewPane(PaneId, ClientOrTabIndex),
@@ -84,6 +85,10 @@ pub(crate) enum ScreenInstruction {
     Copy(ClientId),
     AddClient(ClientId),
     RemoveClient(ClientId),
+    AddOverlay(Overlay, ClientId),
+    RemoveOverlay(ClientId),
+    ConfirmPrompt(ClientId),
+    DenyPrompt(ClientId),
 }
 
 impl From<&ScreenInstruction> for ScreenContext {
@@ -154,6 +159,10 @@ impl From<&ScreenInstruction> for ScreenContext {
             ScreenInstruction::ToggleTab(..) => ScreenContext::ToggleTab,
             ScreenInstruction::AddClient(..) => ScreenContext::AddClient,
             ScreenInstruction::RemoveClient(..) => ScreenContext::RemoveClient,
+            ScreenInstruction::AddOverlay(..) => ScreenContext::AddOverlay,
+            ScreenInstruction::RemoveOverlay(..) => ScreenContext::RemoveOverlay,
+            ScreenInstruction::ConfirmPrompt(..) => ScreenContext::ConfirmPrompt,
+            ScreenInstruction::DenyPrompt(..) => ScreenContext::DenyPrompt,
         }
     }
 }
@@ -169,6 +178,8 @@ pub(crate) struct Screen {
     tabs: BTreeMap<usize, Tab>,
     /// The full size of this [`Screen`].
     size: Size,
+    /// The overlay that is drawn on top of [`Pane`]'s', [`Tab`]'s and the [`Screen`]
+    overlay: OverlayWindow,
     /// The indices of this [`Screen`]'s active [`Tab`]s.
     active_tab_indices: BTreeMap<ClientId, usize>,
     tab_history: BTreeMap<ClientId, Vec<usize>>,
@@ -193,6 +204,7 @@ impl Screen {
             colors: client_attributes.palette,
             active_tab_indices: BTreeMap::new(),
             tabs: BTreeMap::new(),
+            overlay: OverlayWindow::default(),
             tab_history: BTreeMap::new(),
             mode_info,
             draw_pane_frames,
@@ -347,9 +359,12 @@ impl Screen {
     pub fn render(&mut self) {
         let mut output = Output::default();
         let mut tabs_to_close = vec![];
+        let size = self.size;
+        let overlay = self.overlay.clone();
         for (tab_index, tab) in self.tabs.iter_mut() {
-            if tab.get_active_pane().is_some() {
-                tab.render(&mut output);
+            if tab.has_active_panes() {
+                let vte_overlay = overlay.generate_overlay(size);
+                tab.render(&mut output, Some(vte_overlay));
             } else {
                 tabs_to_close.push(*tab_index);
             }
@@ -393,6 +408,11 @@ impl Screen {
         }
     }
 
+    /// Returns a mutable reference to this [`Screen`]'s active [`Overlays`].
+    pub fn get_active_overlays_mut(&mut self) -> &mut Vec<Overlay> {
+        &mut self.overlay.overlay_stack
+    }
+
     /// Returns a mutable reference to this [`Screen`]'s indexed [`Tab`].
     pub fn get_indexed_tab_mut(&mut self, tab_index: usize) -> Option<&mut Tab> {
         self.get_tabs_mut().get_mut(&tab_index)
@@ -511,20 +531,20 @@ impl Screen {
         {
             self.get_active_tab_mut(client_id)
                 .unwrap()
-                .clear_active_terminal_scroll();
+                .clear_active_terminal_scroll(client_id);
         }
         self.colors = mode_info.palette;
         self.mode_info = mode_info;
         for tab in self.tabs.values_mut() {
             tab.mode_info = self.mode_info.clone();
-            tab.mark_active_pane_for_rerender();
+            tab.mark_active_pane_for_rerender(client_id);
         }
     }
     pub fn move_focus_left_or_previous_tab(&mut self, client_id: ClientId) {
         if !self
             .get_active_tab_mut(client_id)
             .unwrap()
-            .move_focus_left()
+            .move_focus_left(client_id)
         {
             self.switch_tab_prev(client_id);
         }
@@ -533,7 +553,7 @@ impl Screen {
         if !self
             .get_active_tab_mut(client_id)
             .unwrap()
-            .move_focus_right()
+            .move_focus_right(client_id)
         {
             self.switch_tab_next(client_id);
         }
@@ -560,7 +580,7 @@ pub(crate) fn screen_thread_main(
     config_options: Box<Options>,
 ) {
     let capabilities = config_options.simplified_ui;
-    let draw_pane_frames = !config_options.no_pane_frames.unwrap_or_default();
+    let draw_pane_frames = config_options.pane_frames.unwrap_or(true);
 
     let mut screen = Screen::new(
         bus,
@@ -597,10 +617,13 @@ pub(crate) fn screen_thread_main(
             ScreenInstruction::NewPane(pid, client_or_tab_index) => {
                 match client_or_tab_index {
                     ClientOrTabIndex::ClientId(client_id) => {
-                        screen.get_active_tab_mut(client_id).unwrap().new_pane(pid);
+                        screen
+                            .get_active_tab_mut(client_id)
+                            .unwrap()
+                            .new_pane(pid, Some(client_id));
                     }
                     ClientOrTabIndex::TabIndex(tab_index) => {
-                        screen.tabs.get_mut(&tab_index).unwrap().new_pane(pid);
+                        screen.tabs.get_mut(&tab_index).unwrap().new_pane(pid, None);
                     }
                 };
                 screen
@@ -616,7 +639,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .horizontal_split(pid);
+                    .horizontal_split(pid, client_id);
                 screen
                     .bus
                     .senders
@@ -630,7 +653,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .vertical_split(pid);
+                    .vertical_split(pid, client_id);
                 screen
                     .bus
                     .senders
@@ -644,26 +667,38 @@ pub(crate) fn screen_thread_main(
                 let active_tab = screen.get_active_tab_mut(client_id).unwrap();
                 match active_tab.is_sync_panes_active() {
                     true => active_tab.write_to_terminals_on_current_tab(bytes),
-                    false => active_tab.write_to_active_terminal(bytes),
+                    false => active_tab.write_to_active_terminal(bytes, client_id),
                 }
             }
             ScreenInstruction::ResizeLeft(client_id) => {
-                screen.get_active_tab_mut(client_id).unwrap().resize_left();
+                screen
+                    .get_active_tab_mut(client_id)
+                    .unwrap()
+                    .resize_left(client_id);
 
                 screen.render();
             }
             ScreenInstruction::ResizeRight(client_id) => {
-                screen.get_active_tab_mut(client_id).unwrap().resize_right();
+                screen
+                    .get_active_tab_mut(client_id)
+                    .unwrap()
+                    .resize_right(client_id);
 
                 screen.render();
             }
             ScreenInstruction::ResizeDown(client_id) => {
-                screen.get_active_tab_mut(client_id).unwrap().resize_down();
+                screen
+                    .get_active_tab_mut(client_id)
+                    .unwrap()
+                    .resize_down(client_id);
 
                 screen.render();
             }
             ScreenInstruction::ResizeUp(client_id) => {
-                screen.get_active_tab_mut(client_id).unwrap().resize_up();
+                screen
+                    .get_active_tab_mut(client_id)
+                    .unwrap()
+                    .resize_up(client_id);
 
                 screen.render();
             }
@@ -671,7 +706,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .resize_increase();
+                    .resize_increase(client_id);
 
                 screen.render();
             }
@@ -679,12 +714,15 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .resize_decrease();
+                    .resize_decrease(client_id);
 
                 screen.render();
             }
             ScreenInstruction::SwitchFocus(client_id) => {
-                screen.get_active_tab_mut(client_id).unwrap().move_focus();
+                screen
+                    .get_active_tab_mut(client_id)
+                    .unwrap()
+                    .move_focus(client_id);
 
                 screen.render();
             }
@@ -692,7 +730,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .focus_next_pane();
+                    .focus_next_pane(client_id);
 
                 screen.render();
             }
@@ -700,7 +738,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .focus_previous_pane();
+                    .focus_previous_pane(client_id);
 
                 screen.render();
             }
@@ -708,7 +746,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_focus_left();
+                    .move_focus_left(client_id);
 
                 screen.render();
             }
@@ -726,7 +764,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_focus_down();
+                    .move_focus_down(client_id);
 
                 screen.render();
             }
@@ -734,7 +772,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_focus_right();
+                    .move_focus_right(client_id);
 
                 screen.render();
             }
@@ -752,7 +790,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_focus_up();
+                    .move_focus_up(client_id);
 
                 screen.render();
             }
@@ -760,7 +798,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .scroll_active_terminal_up();
+                    .scroll_active_terminal_up(client_id);
 
                 screen.render();
             }
@@ -768,7 +806,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_active_pane();
+                    .move_active_pane(client_id);
 
                 screen.render();
             }
@@ -776,7 +814,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_active_pane_down();
+                    .move_active_pane_down(client_id);
 
                 screen.render();
             }
@@ -784,7 +822,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_active_pane_up();
+                    .move_active_pane_up(client_id);
 
                 screen.render();
             }
@@ -792,7 +830,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_active_pane_right();
+                    .move_active_pane_right(client_id);
 
                 screen.render();
             }
@@ -800,7 +838,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .move_active_pane_left();
+                    .move_active_pane_left(client_id);
 
                 screen.render();
             }
@@ -816,7 +854,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .scroll_active_terminal_down();
+                    .scroll_active_terminal_down(client_id);
 
                 screen.render();
             }
@@ -832,7 +870,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .scroll_active_terminal_to_bottom();
+                    .scroll_active_terminal_to_bottom(client_id);
 
                 screen.render();
             }
@@ -840,7 +878,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .scroll_active_terminal_up_page();
+                    .scroll_active_terminal_up_page(client_id);
 
                 screen.render();
             }
@@ -848,7 +886,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .scroll_active_terminal_down_page();
+                    .scroll_active_terminal_down_page(client_id);
 
                 screen.render();
             }
@@ -856,7 +894,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .scroll_active_terminal_up_half_page();
+                    .scroll_active_terminal_up_half_page(client_id);
 
                 screen.render();
             }
@@ -864,7 +902,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .scroll_active_terminal_down_half_page();
+                    .scroll_active_terminal_down_half_page(client_id);
 
                 screen.render();
             }
@@ -872,7 +910,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .clear_active_terminal_scroll();
+                    .clear_active_terminal_scroll(client_id);
 
                 screen.render();
             }
@@ -880,7 +918,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .close_focused_pane();
+                    .close_focused_pane(client_id);
                 screen.update_tabs(); // update_tabs eventually calls render through the plugin thread
             }
             ScreenInstruction::SetSelectable(id, selectable, tab_index) => {
@@ -917,7 +955,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .toggle_active_pane_fullscreen();
+                    .toggle_active_pane_fullscreen(client_id);
                 screen.update_tabs();
 
                 screen.render();
@@ -1011,7 +1049,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .handle_left_click(&point);
+                    .handle_left_click(&point, client_id);
 
                 screen.render();
             }
@@ -1019,7 +1057,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .handle_right_click(&point);
+                    .handle_right_click(&point, client_id);
 
                 screen.render();
             }
@@ -1027,7 +1065,7 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .handle_mouse_release(&point);
+                    .handle_mouse_release(&point, client_id);
 
                 screen.render();
             }
@@ -1035,12 +1073,15 @@ pub(crate) fn screen_thread_main(
                 screen
                     .get_active_tab_mut(client_id)
                     .unwrap()
-                    .handle_mouse_hold(&point);
+                    .handle_mouse_hold(&point, client_id);
 
                 screen.render();
             }
             ScreenInstruction::Copy(client_id) => {
-                screen.get_active_tab(client_id).unwrap().copy_selection();
+                screen
+                    .get_active_tab(client_id)
+                    .unwrap()
+                    .copy_selection(client_id);
 
                 screen.render();
             }
@@ -1067,6 +1108,45 @@ pub(crate) fn screen_thread_main(
 
                 screen.render();
             }
+            ScreenInstruction::AddOverlay(overlay, _client_id) => {
+                screen.get_active_overlays_mut().pop();
+                screen.get_active_overlays_mut().push(overlay);
+                screen
+                    .bus
+                    .senders
+                    .send_to_server(ServerInstruction::UnblockInputThread)
+                    .unwrap();
+            }
+            ScreenInstruction::RemoveOverlay(_client_id) => {
+                screen.get_active_overlays_mut().pop();
+                screen.render();
+                screen
+                    .bus
+                    .senders
+                    .send_to_server(ServerInstruction::UnblockInputThread)
+                    .unwrap();
+            }
+            ScreenInstruction::ConfirmPrompt(_client_id) => {
+                let overlay = screen.get_active_overlays_mut().pop();
+                let instruction = overlay.map(|o| o.prompt_confirm()).flatten();
+                if let Some(instruction) = instruction {
+                    screen.bus.senders.send_to_server(*instruction).unwrap();
+                }
+                screen
+                    .bus
+                    .senders
+                    .send_to_server(ServerInstruction::UnblockInputThread)
+                    .unwrap();
+            }
+            ScreenInstruction::DenyPrompt(_client_id) => {
+                screen.get_active_overlays_mut().pop();
+                screen.render();
+                screen
+                    .bus
+                    .senders
+                    .send_to_server(ServerInstruction::UnblockInputThread)
+                    .unwrap();
+            }
         }
     }
 }
