@@ -1,13 +1,17 @@
-use std::collections::{HashMap, HashSet};
-use std::fs;
-use std::path::PathBuf;
-use std::process;
-use std::str::FromStr;
-use std::sync::{mpsc::Sender, Arc, Mutex};
-use std::thread;
-use std::time::{Duration, Instant};
-
+use highway::{HighwayHash, PortableHash};
+use log::{debug, info, warn};
 use serde::{de::DeserializeOwned, Serialize};
+use std::{
+    collections::{HashMap, HashSet},
+    fs,
+    path::{Path, PathBuf},
+    process,
+    str::FromStr,
+    sync::{mpsc::Sender, Arc, Mutex},
+    thread,
+    time::{Duration, Instant},
+};
+use url::Url;
 use wasmer::{
     imports, ChainableNamedResolver, Function, ImportObject, Instance, Module, Store, Value,
     WasmerEnv,
@@ -16,18 +20,28 @@ use wasmer_wasi::{Pipe, WasiEnv, WasiState};
 use zellij_tile::data::{Event, EventType, PluginIds};
 
 use crate::{
+    logging_pipe::LoggingPipe,
     panes::PaneId,
-    pty::PtyInstruction,
+    pty::{ClientOrTabIndex, PtyInstruction},
     screen::ScreenInstruction,
     thread_bus::{Bus, ThreadSenders},
 };
-use zellij_utils::errors::{ContextType, PluginContext};
-use zellij_utils::{input::command::TerminalAction, serde, zellij_tile};
+
+use zellij_utils::{
+    consts::{VERSION, ZELLIJ_PROJ_DIR},
+    errors::{ContextType, PluginContext},
+};
+use zellij_utils::{
+    input::command::TerminalAction,
+    input::layout::RunPlugin,
+    input::plugins::{PluginConfig, PluginType, PluginsConfig},
+    serde, zellij_tile,
+};
 
 #[derive(Clone, Debug)]
 pub(crate) enum PluginInstruction {
-    Load(Sender<u32>, PathBuf),
-    Update(Option<u32>, Event), // Focused plugin / broadcast, event data
+    Load(Sender<u32>, RunPlugin, usize), // tx_pid, plugin metadata, tab_index
+    Update(Option<u32>, Event),          // Focused plugin / broadcast, event data
     Render(Sender<String>, u32, usize, usize), // String buffer, plugin id, rows, cols
     Unload(u32),
     Exit,
@@ -48,62 +62,63 @@ impl From<&PluginInstruction> for PluginContext {
 #[derive(WasmerEnv, Clone)]
 pub(crate) struct PluginEnv {
     pub plugin_id: u32,
+    pub plugin: PluginConfig,
     pub senders: ThreadSenders,
     pub wasi_env: WasiEnv,
     pub subscriptions: Arc<Mutex<HashSet<EventType>>>,
+    pub tab_index: usize,
+    plugin_own_data_dir: PathBuf,
 }
 
 // Thread main --------------------------------------------------------------------------------------------------------
-pub(crate) fn wasm_thread_main(bus: Bus<PluginInstruction>, store: Store, data_dir: PathBuf) {
+pub(crate) fn wasm_thread_main(
+    bus: Bus<PluginInstruction>,
+    store: Store,
+    data_dir: PathBuf,
+    plugins: PluginsConfig,
+) {
+    info!("Wasm main thread starts");
+
     let mut plugin_id = 0;
     let mut plugin_map = HashMap::new();
+    let plugin_dir = data_dir.join("plugins/");
+    let plugin_global_data_dir = plugin_dir.join("data");
+    fs::create_dir_all(plugin_global_data_dir.as_path()).unwrap();
+
+    for plugin in plugins.iter() {
+        if let PluginType::Headless = plugin.run {
+            let (instance, plugin_env) = start_plugin(
+                plugin_id,
+                plugin,
+                0,
+                &bus,
+                &store,
+                &data_dir,
+                &plugin_global_data_dir,
+            );
+            plugin_map.insert(plugin_id, (instance, plugin_env));
+            plugin_id += 1;
+        }
+    }
+
     loop {
         let (event, mut err_ctx) = bus.recv().expect("failed to receive event on channel");
         err_ctx.add_call(ContextType::Plugin((&event).into()));
         match event {
-            PluginInstruction::Load(pid_tx, path) => {
-                let plugin_dir = data_dir.join("plugins/");
-                let wasm_bytes = fs::read(&path)
-                    .or_else(|_| fs::read(&path.with_extension("wasm")))
-                    .or_else(|_| fs::read(&plugin_dir.join(&path).with_extension("wasm")))
-                    .unwrap_or_else(|_| panic!("cannot find plugin {}", &path.display()));
+            PluginInstruction::Load(pid_tx, run, tab_index) => {
+                let plugin = plugins
+                    .get(&run)
+                    .unwrap_or_else(|| panic!("Plugin {:?} could not be resolved", run));
 
-                // FIXME: Cache this compiled module on disk. I could use `(de)serialize_to_file()` for that
-                let module = Module::new(&store, &wasm_bytes).unwrap();
-
-                let output = Pipe::new();
-                let input = Pipe::new();
-                let mut wasi_env = WasiState::new("Zellij")
-                    .env("CLICOLOR_FORCE", "1")
-                    .preopen(|p| {
-                        p.directory(".") // FIXME: Change this to a more meaningful dir
-                            .alias(".")
-                            .read(true)
-                            .write(true)
-                            .create(true)
-                    })
-                    .unwrap()
-                    .stdin(Box::new(input))
-                    .stdout(Box::new(output))
-                    .finalize()
-                    .unwrap();
-
-                let wasi = wasi_env.import_object(&module).unwrap();
-
-                let plugin_env = PluginEnv {
+                let (instance, plugin_env) = start_plugin(
                     plugin_id,
-                    senders: bus.senders.clone(),
-                    wasi_env,
-                    subscriptions: Arc::new(Mutex::new(HashSet::new())),
-                };
-
-                let zellij = zellij_exports(&store, &plugin_env);
-                let instance = Instance::new(&module, &zellij.chain_back(wasi)).unwrap();
-
-                let start = instance.exports.get_function("_start").unwrap();
-
-                // This eventually calls the `.load()` method
-                start.call(&[]).unwrap();
+                    &plugin,
+                    tab_index,
+                    &bus,
+                    &store,
+                    &data_dir,
+                    &plugin_global_data_dir,
+                );
 
                 plugin_map.insert(plugin_id, (instance, plugin_env));
                 pid_tx.send(plugin_id).unwrap();
@@ -123,20 +138,109 @@ pub(crate) fn wasm_thread_main(bus: Bus<PluginInstruction>, store: Store, data_d
                 drop(bus.senders.send_to_screen(ScreenInstruction::Render));
             }
             PluginInstruction::Render(buf_tx, pid, rows, cols) => {
-                let (instance, plugin_env) = plugin_map.get(&pid).unwrap();
+                if rows == 0 || cols == 0 {
+                    buf_tx.send(String::new()).unwrap();
+                } else {
+                    let (instance, plugin_env) = plugin_map.get(&pid).unwrap();
+                    let render = instance.exports.get_function("render").unwrap();
 
-                let render = instance.exports.get_function("render").unwrap();
+                    render
+                        .call(&[Value::I32(rows as i32), Value::I32(cols as i32)])
+                        .unwrap();
 
-                render
-                    .call(&[Value::I32(rows as i32), Value::I32(cols as i32)])
-                    .unwrap();
-
-                buf_tx.send(wasi_read_string(&plugin_env.wasi_env)).unwrap();
+                    buf_tx.send(wasi_read_string(&plugin_env.wasi_env)).unwrap();
+                }
+            }
+            PluginInstruction::Unload(pid) => {
+                info!("Bye from plugin {}", &pid);
+                // TODO: remove plugin's own data directory
+                drop(plugin_map.remove(&pid));
             }
-            PluginInstruction::Unload(pid) => drop(plugin_map.remove(&pid)),
             PluginInstruction::Exit => break,
         }
     }
+    info!("wasm main thread exits");
+    fs::remove_dir_all(plugin_global_data_dir.as_path()).unwrap();
+}
+
+fn start_plugin(
+    plugin_id: u32,
+    plugin: &PluginConfig,
+    tab_index: usize,
+    bus: &Bus<PluginInstruction>,
+    store: &Store,
+    data_dir: &Path,
+    plugin_global_data_dir: &Path,
+) -> (Instance, PluginEnv) {
+    if plugin._allow_exec_host_cmd {
+        info!(
+            "Plugin({:?}) is able to run any host command, this may lead to some security issues!",
+            plugin.path
+        );
+    }
+
+    let wasm_bytes = plugin
+        .resolve_wasm_bytes(&data_dir.join("plugins/"))
+        .unwrap_or_else(|| panic!("Cannot resolve wasm bytes for plugin {:?}", plugin));
+
+    let hash: String = PortableHash::default()
+        .hash256(&wasm_bytes)
+        .iter()
+        .map(ToString::to_string)
+        .collect();
+
+    let cached_path = ZELLIJ_PROJ_DIR.cache_dir().join(&hash);
+
+    let module = unsafe {
+        Module::deserialize_from_file(store, &cached_path).unwrap_or_else(|_| {
+            let m = Module::new(store, &wasm_bytes).unwrap();
+            fs::create_dir_all(ZELLIJ_PROJ_DIR.cache_dir()).unwrap();
+            m.serialize_to_file(&cached_path).unwrap();
+            m
+        })
+    };
+
+    let output = Pipe::new();
+    let input = Pipe::new();
+    let stderr = LoggingPipe::new(&plugin.location.to_string(), plugin_id);
+    let plugin_own_data_dir = plugin_global_data_dir.join(Url::from(&plugin.location).to_string());
+    fs::create_dir_all(&plugin_own_data_dir).unwrap();
+
+    let mut wasi_env = WasiState::new("Zellij")
+        .env("CLICOLOR_FORCE", "1")
+        .map_dir("/host", ".")
+        .unwrap()
+        .map_dir("/data", plugin_own_data_dir.as_path())
+        .unwrap()
+        .stdin(Box::new(input))
+        .stdout(Box::new(output))
+        .stderr(Box::new(stderr))
+        .finalize()
+        .unwrap();
+
+    let wasi = wasi_env.import_object(&module).unwrap();
+    let mut plugin = plugin.clone();
+    plugin.set_tab_index(tab_index);
+
+    let plugin_env = PluginEnv {
+        plugin_id,
+        plugin,
+        senders: bus.senders.clone(),
+        wasi_env,
+        subscriptions: Arc::new(Mutex::new(HashSet::new())),
+        plugin_own_data_dir,
+        tab_index,
+    };
+
+    let zellij = zellij_exports(store, &plugin_env);
+    let instance = Instance::new(&module, &zellij.chain_back(wasi)).unwrap();
+
+    let start = instance.exports.get_function("_start").unwrap();
+
+    // This eventually calls the `.load()` method
+    start.call(&[]).unwrap();
+
+    (instance, plugin_env)
 }
 
 // Plugin API ---------------------------------------------------------------------------------------------------------
@@ -156,13 +260,13 @@ pub(crate) fn zellij_exports(store: &Store, plugin_env: &PluginEnv) -> ImportObj
     zellij_export! {
         host_subscribe,
         host_unsubscribe,
-        host_set_invisible_borders,
-        host_set_fixed_height,
-        host_set_fixed_width,
         host_set_selectable,
         host_get_plugin_ids,
+        host_get_zellij_version,
         host_open_file,
+        host_switch_tab_to,
         host_set_timeout,
+        host_exec_cmd,
     }
 }
 
@@ -179,47 +283,25 @@ fn host_unsubscribe(plugin_env: &PluginEnv) {
 }
 
 fn host_set_selectable(plugin_env: &PluginEnv, selectable: i32) {
-    let selectable = selectable != 0;
-    plugin_env
-        .senders
-        .send_to_screen(ScreenInstruction::SetSelectable(
-            PaneId::Plugin(plugin_env.plugin_id),
-            selectable,
-        ))
-        .unwrap()
-}
-
-fn host_set_fixed_height(plugin_env: &PluginEnv, fixed_height: i32) {
-    let fixed_height = fixed_height as usize;
-    plugin_env
-        .senders
-        .send_to_screen(ScreenInstruction::SetFixedHeight(
-            PaneId::Plugin(plugin_env.plugin_id),
-            fixed_height,
-        ))
-        .unwrap()
-}
-
-fn host_set_fixed_width(plugin_env: &PluginEnv, fixed_width: i32) {
-    let fixed_width = fixed_width as usize;
-    plugin_env
-        .senders
-        .send_to_screen(ScreenInstruction::SetFixedWidth(
-            PaneId::Plugin(plugin_env.plugin_id),
-            fixed_width,
-        ))
-        .unwrap()
-}
-
-fn host_set_invisible_borders(plugin_env: &PluginEnv, invisible_borders: i32) {
-    let invisible_borders = invisible_borders != 0;
-    plugin_env
-        .senders
-        .send_to_screen(ScreenInstruction::SetInvisibleBorders(
-            PaneId::Plugin(plugin_env.plugin_id),
-            invisible_borders,
-        ))
-        .unwrap()
+    match plugin_env.plugin.run {
+        PluginType::Pane(Some(tab_index)) => {
+            let selectable = selectable != 0;
+            plugin_env
+                .senders
+                .send_to_screen(ScreenInstruction::SetSelectable(
+                    PaneId::Plugin(plugin_env.plugin_id),
+                    selectable,
+                    tab_index,
+                ))
+                .unwrap()
+        }
+        _ => {
+            debug!(
+                "{} - Calling method 'host_set_selectable' does nothing for headless plugins",
+                plugin_env.plugin.location
+            )
+        }
+    }
 }
 
 fn host_get_plugin_ids(plugin_env: &PluginEnv) {
@@ -230,13 +312,25 @@ fn host_get_plugin_ids(plugin_env: &PluginEnv) {
     wasi_write_object(&plugin_env.wasi_env, &ids);
 }
 
+fn host_get_zellij_version(plugin_env: &PluginEnv) {
+    wasi_write_string(&plugin_env.wasi_env, VERSION);
+}
+
 fn host_open_file(plugin_env: &PluginEnv) {
     let path: PathBuf = wasi_read_object(&plugin_env.wasi_env);
     plugin_env
         .senders
-        .send_to_pty(PtyInstruction::SpawnTerminal(Some(
-            TerminalAction::OpenFile(path),
-        )))
+        .send_to_pty(PtyInstruction::SpawnTerminal(
+            Some(TerminalAction::OpenFile(path)),
+            ClientOrTabIndex::TabIndex(plugin_env.tab_index),
+        ))
+        .unwrap();
+}
+
+fn host_switch_tab_to(plugin_env: &PluginEnv, tab_idx: u32) {
+    plugin_env
+        .senders
+        .send_to_screen(ScreenInstruction::GoToTab(tab_idx, None)) // this is a hack, we should be able to return the client id here
         .unwrap();
 }
 
@@ -269,9 +363,26 @@ fn host_set_timeout(plugin_env: &PluginEnv, secs: f64) {
     });
 }
 
+fn host_exec_cmd(plugin_env: &PluginEnv) {
+    let mut cmdline: Vec<String> = wasi_read_object(&plugin_env.wasi_env);
+    let command = cmdline.remove(0);
+
+    // Bail out if we're forbidden to run command
+    if !plugin_env.plugin._allow_exec_host_cmd {
+        warn!("This plugin isn't allow to run command in host side, skip running this command: '{cmd} {args}'.",
+        	cmd = command, args = cmdline.join(" "));
+        return;
+    }
+
+    // Here, we don't wait the command to finish
+    process::Command::new(command)
+        .args(cmdline)
+        .spawn()
+        .unwrap();
+}
+
 // Helper Functions ---------------------------------------------------------------------------------------------------
 
-// FIXME: Unwrap city
 pub fn wasi_read_string(wasi_env: &WasiEnv) -> String {
     let mut state = wasi_env.state();
     let wasi_file = state.fs.stdout_mut().unwrap().as_mut().unwrap();
