@@ -27,6 +27,11 @@ pub fn get_plugin_ids() -> PluginIds {
     object_from_stdin().unwrap()
 }
 
+pub fn get_zellij_version() -> String {
+    unsafe { host_get_zellij_version() };
+    object_from_stdin().unwrap()
+}
+
 // Host Functions
 
 pub fn open_file(path: &Path) {
@@ -34,6 +39,10 @@ pub fn open_file(path: &Path) {
     unsafe { host_open_file() };
 }
 
+pub fn switch_tab_to(tab_idx: u32) {
+    unsafe { host_switch_tab_to(tab_idx) };
+}
+
 pub fn set_timeout(secs: f64) {
     unsafe { host_set_timeout(secs) };
 }
@@ -62,7 +71,9 @@ extern "C" {
     fn host_unsubscribe();
     fn host_set_selectable(selectable: i32);
     fn host_get_plugin_ids();
+    fn host_get_zellij_version();
     fn host_open_file();
+    fn host_switch_tab_to(tab_idx: u32);
     fn host_set_timeout(secs: f64);
     fn host_exec_cmd();
 }
