@@ -5,7 +5,7 @@ use std::io::{self, Read};
 use std::path::{Path, PathBuf};
 use thiserror::Error;
 
-use serde::Deserialize;
+use serde::{Deserialize, Serialize};
 use std::convert::{TryFrom, TryInto};
 
 use super::keybinds::{Keybinds, KeybindsFromYaml};
@@ -20,7 +20,7 @@ const DEFAULT_CONFIG_FILE_NAME: &str = "config.yaml";
 type ConfigResult = Result<Config, ConfigError>;
 
 /// Intermediate deserialization config struct
-#[derive(Clone, Debug, Deserialize)]
+#[derive(Clone, Default, Debug, Deserialize, Serialize, PartialEq)]
 pub struct ConfigFromYaml {
     #[serde(flatten)]
     pub options: Option<Options>,
@@ -151,6 +151,18 @@ impl Config {
         let cfg = String::from_utf8(setup::DEFAULT_CONFIG.to_vec())?;
         Self::from_yaml(cfg.as_str())
     }
+
+    /// Merges two Config structs into one Config struct
+    /// `other` overrides `self`.
+    pub fn merge(&self, other: Self) -> Self {
+        //let themes = if let Some()
+        Self {
+            keybinds: self.keybinds.merge_keybinds(other.keybinds),
+            options: self.options.merge(other.options),
+            themes: None,
+            plugins: self.plugins.merge(other.plugins),
+        }
+    }
 }
 
 impl TryFrom<ConfigFromYaml> for Config {
