@@ -68,6 +68,7 @@ pub enum ClientToServerMsg {
     AttachClient(ClientAttributes, Options),
     Action(Action),
     ClientExited,
+    KillSession,
 }
 
 // Types of messages sent from the server to the client
@@ -103,7 +104,7 @@ impl Display for ExitReason {
                 f,
                 "Session attached to another client. Use --force flag to force connect."
             ),
-            Self::Error(e) => write!(f, "Error occured in server:\n{}", e),
+            Self::Error(e) => write!(f, "Error occurred in server:\n{}", e),
         }
     }
 }
